@@ -342,7 +342,7 @@ class TestGenericSwitchDriver(unittest.TestCase):
                           driver.delete_port_postcommit,
                           mock_context)
 
-    def test_delete_port_potcommit_unknown_switch(self, m_list):
+    def test_delete_port_postcommit_unknown_switch(self, m_list):
         driver = gsm.GenericSwitchDriver()
         driver.initialize()
         mock_context = mock.create_autospec(driver_context.PortContext)
@@ -408,12 +408,14 @@ class TestGenericSwitchDriver(unittest.TestCase):
                                  },
                                 'binding:vnic_type': 'baremetal',
                                 'id': '123',
-                                'binding:vif_type': 'unbound'}
+                                'binding:vif_type': 'unbound',
+                                'status': 'DOWN'}
         mock_context.original = {'binding:profile': {},
                                  'binding:vnic_type': 'baremetal',
                                  'id': '123',
                                  'binding:vif_type': 'unbound'}
         driver.update_port_postcommit(mock_context)
+        self.switch_mock.plug_port_to_network.assert_not_called()
         self.assertFalse(m_pc.called)
 
     @mock.patch.object(provisioning_blocks, 'provisioning_complete')
@@ -433,12 +435,14 @@ class TestGenericSwitchDriver(unittest.TestCase):
                                  },
                                 'binding:vnic_type': 'mcvtap',
                                 'id': '123',
-                                'binding:vif_type': 'other'}
+                                'binding:vif_type': 'other',
+                                'status': 'DOWN'}
         mock_context.original = {'binding:profile': {},
                                  'binding:vnic_type': 'mcvtap',
                                  'id': '123',
                                  'binding:vif_type': 'unbound'}
         driver.update_port_postcommit(mock_context)
+        self.switch_mock.plug_port_to_network.assert_not_called()
         self.assertFalse(m_pc.called)
 
     @mock.patch.object(provisioning_blocks, 'provisioning_complete')
@@ -450,12 +454,14 @@ class TestGenericSwitchDriver(unittest.TestCase):
         mock_context.current = {'binding:profile': {},
                                 'binding:vnic_type': 'baremetal',
                                 'id': '123',
-                                'binding:vif_type': 'other'}
+                                'binding:vif_type': 'other',
+                                'status': 'DOWN'}
         mock_context.original = {'binding:profile': {},
                                  'binding:vnic_type': 'baremetal',
                                  'id': '123',
                                  'binding:vif_type': 'unbound'}
         driver.update_port_postcommit(mock_context)
+        self.switch_mock.plug_port_to_network.assert_not_called()
         self.assertFalse(m_pc.called)
 
     @mock.patch.object(provisioning_blocks, 'provisioning_complete')
@@ -475,12 +481,14 @@ class TestGenericSwitchDriver(unittest.TestCase):
                                  },
                                 'binding:vnic_type': 'baremetal',
                                 'id': '123',
-                                'binding:vif_type': 'other'}
+                                'binding:vif_type': 'other',
+                                'status': 'DOWN'}
         mock_context.original = {'binding:profile': {},
                                  'binding:vnic_type': 'baremetal',
                                  'id': '123',
                                  'binding:vif_type': 'unbound'}
         driver.update_port_postcommit(mock_context)
+        self.switch_mock.plug_port_to_network.assert_not_called()
         self.assertFalse(m_pc.called)
 
     @mock.patch.object(provisioning_blocks, 'provisioning_complete')
@@ -500,13 +508,20 @@ class TestGenericSwitchDriver(unittest.TestCase):
                                  },
                                 'binding:vnic_type': 'baremetal',
                                 'id': '123',
-                                'binding:vif_type': 'other'}
+                                'binding:vif_type': 'other',
+                                'status': 'DOWN'}
         mock_context.original = {'binding:profile': {},
                                  'binding:vnic_type': 'baremetal',
                                  'id': '123',
                                  'binding:vif_type': 'unbound'}
+        mock_context.network = mock.Mock()
+        mock_context.network.current = {
+            'provider:segmentation_id': 42,
+            'provider:physical_network': 'physnet1'
+        }
         driver.update_port_postcommit(mock_context)
-        self.switch_mock.plug_port_to_network.assert_not_called()
+        self.switch_mock.plug_port_to_network.assert_called_once_with(
+            2222, 42)
         m_pc.assert_called_once_with(mock_context._plugin_context,
                                      mock_context.current['id'],
                                      resources.PORT,
@@ -535,13 +550,23 @@ class TestGenericSwitchDriver(unittest.TestCase):
                                  },
                                 'binding:vnic_type': 'baremetal',
                                 'id': '123',
-                                'binding:vif_type': 'other'}
+                                'binding:vif_type': 'other',
+                                'status': 'DOWN'}
         mock_context.original = {'binding:profile': {},
                                  'binding:vnic_type': 'baremetal',
                                  'id': '123',
                                  'binding:vif_type': 'unbound'}
+        mock_context.network = mock.Mock()
+        mock_context.network.current = {
+            'provider:segmentation_id': 42,
+            'provider:physical_network': 'physnet1'
+        }
         driver.update_port_postcommit(mock_context)
-        self.switch_mock.plug_port_to_network.assert_not_called()
+        self.switch_mock.plug_port_to_network.assert_has_calls(
+            [mock.call(2222, 42),
+             mock.call(3333, 42)]
+        )
+        self.switch_mock.plug_bond_to_network.assert_not_called()
         m_pc.assert_has_calls([mock.call(mock_context._plugin_context,
                                          mock_context.current['id'],
                                          resources.PORT,
@@ -574,18 +599,97 @@ class TestGenericSwitchDriver(unittest.TestCase):
                                  },
                                 'binding:vnic_type': 'baremetal',
                                 'id': '123',
-                                'binding:vif_type': 'other'}
+                                'binding:vif_type': 'other',
+                                'status': 'DOWN'}
         mock_context.original = {'binding:profile': {},
                                  'binding:vnic_type': 'baremetal',
                                  'id': '123',
                                  'binding:vif_type': 'unbound'}
+        mock_context.network = mock.Mock()
+        mock_context.network.current = {
+            'provider:segmentation_id': 42,
+            'provider:physical_network': 'physnet1'
+        }
         driver.update_port_postcommit(mock_context)
-        self.switch_mock.plug_bond_to_network.assert_not_called()
+        self.switch_mock.plug_bond_to_network.assert_has_calls(
+            [mock.call(2222, 42),
+             mock.call(3333, 42)]
+        )
+        self.switch_mock.plug_port_to_network.assert_not_called()
         m_pc.assert_has_calls([mock.call(mock_context._plugin_context,
                                          mock_context.current['id'],
                                          resources.PORT,
                                          'GENERICSWITCH')])
 
+    @mock.patch.object(provisioning_blocks, 'provisioning_complete')
+    def test_update_port_postcommit_with_physnet(self, m_pc, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.PortContext)
+        mock_context._plugin_context = mock.MagicMock()
+        mock_context.current = {'binding:profile':
+                                {'local_link_information':
+                                    [
+                                        {
+                                            'switch_info': 'foo',
+                                            'port_id': 2222
+                                        }
+                                    ]
+                                 },
+                                'binding:vnic_type': 'baremetal',
+                                'id': '123',
+                                'binding:vif_type': 'other',
+                                'status': 'DOWN'}
+        mock_context.original = {'binding:profile': {},
+                                 'binding:vnic_type': 'baremetal',
+                                 'id': '123',
+                                 'binding:vif_type': 'unbound'}
+        mock_context.network = mock.Mock()
+        mock_context.network.current = {
+            'provider:physical_network': 'physnet1'
+        }
+        self.switch_mock._get_physical_networks.return_value = ['physnet1']
+
+        driver.update_port_postcommit(mock_context)
+        self.switch_mock.plug_port_to_network.assert_called_once_with(
+            2222, 1)
+        m_pc.assert_called_once_with(mock_context._plugin_context,
+                                     mock_context.current['id'],
+                                     resources.PORT,
+                                     'GENERICSWITCH')
+
+    @mock.patch.object(provisioning_blocks, 'provisioning_complete')
+    def test_update_port_postcommit_with_different_physnet(self, m_pc, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.PortContext)
+        mock_context._plugin_context = mock.MagicMock()
+        mock_context.current = {'binding:profile':
+                                {'local_link_information':
+                                    [
+                                        {
+                                            'switch_info': 'foo',
+                                            'port_id': 2222
+                                        }
+                                    ]
+                                 },
+                                'binding:vnic_type': 'baremetal',
+                                'id': '123',
+                                'binding:vif_type': 'other',
+                                'status': 'DOWN'}
+        mock_context.original = {'binding:profile': {},
+                                 'binding:vnic_type': 'baremetal',
+                                 'id': '123',
+                                 'binding:vif_type': 'unbound'}
+        mock_context.network.current = {
+            'provider:physical_network': 'physnet1'
+        }
+        self.switch_mock._get_physical_networks.return_value = ['physnet2']
+
+        driver.update_port_postcommit(mock_context)
+        self.switch_mock.plug_port_to_network.assert_not_called()
+        m_pc.assert_not_called()
+
     @mock.patch.object(provisioning_blocks, 'provisioning_complete')
     def test_update_port_postcommit_unbind_not_bound(self, m_pc, m_list):
         driver = gsm.GenericSwitchDriver()
@@ -775,13 +879,12 @@ class TestGenericSwitchDriver(unittest.TestCase):
         ]
 
         driver.bind_port(mock_context)
-        self.switch_mock.plug_port_to_network.assert_called_once_with(
-            2222, 1)
         mock_context.set_binding.assert_called_with(123, 'other', {})
         m_apc.assert_called_once_with(mock_context._plugin_context,
                                       mock_context.current['id'],
                                       resources.PORT,
                                       'GENERICSWITCH')
+        self.switch_mock.plug_port_to_network.assert_called()
 
     @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
     def test_bind_portgroup(self, m_apc, m_list):
@@ -815,10 +918,6 @@ class TestGenericSwitchDriver(unittest.TestCase):
         ]
 
         driver.bind_port(mock_context)
-        self.switch_mock.plug_port_to_network.assert_has_calls(
-            [mock.call(2222, 1),
-             mock.call(3333, 1)]
-        )
         mock_context.set_binding.assert_has_calls(
             [mock.call(123, 'other', {})]
         )
@@ -826,6 +925,7 @@ class TestGenericSwitchDriver(unittest.TestCase):
                                           mock_context.current['id'],
                                           resources.PORT,
                                           'GENERICSWITCH')])
+        self.switch_mock.plug_port_to_network.assert_called()
 
     @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
     def test_bind_portgroup_802_3ad(self, m_apc, m_list):
@@ -863,10 +963,6 @@ class TestGenericSwitchDriver(unittest.TestCase):
         ]
 
         driver.bind_port(mock_context)
-        self.switch_mock.plug_bond_to_network.assert_has_calls(
-            [mock.call(2222, 1),
-             mock.call(3333, 1)]
-        )
         mock_context.set_binding.assert_has_calls(
             [mock.call(123, 'other', {})]
         )
@@ -874,6 +970,8 @@ class TestGenericSwitchDriver(unittest.TestCase):
                                           mock_context.current['id'],
                                           resources.PORT,
                                           'GENERICSWITCH')])
+        self.switch_mock.plug_port_to_network.assert_not_called()
+        self.switch_mock.plug_bond_to_network.assert_called()
 
     @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
     def test_bind_port_with_physnet(self, m_apc, m_list):
@@ -904,29 +1002,32 @@ class TestGenericSwitchDriver(unittest.TestCase):
         self.switch_mock._get_physical_networks.return_value = ['physnet1']
 
         driver.bind_port(mock_context)
-        self.switch_mock.plug_port_to_network.assert_called_once_with(
-            2222, 1)
         mock_context.set_binding.assert_called_with(123, 'other', {})
         m_apc.assert_called_once_with(mock_context._plugin_context,
                                       mock_context.current['id'],
                                       resources.PORT,
                                       'GENERICSWITCH')
+        self.switch_mock.plug_port_to_network.assert_called()
 
     @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
-    def test_bind_port_unknown_switch(self, m_apc, m_list):
+    def test_bind_portgroup_port_not_supported(self, m_apc, m_list):
         driver = gsm.GenericSwitchDriver()
         driver.initialize()
+        driver._is_port_supported = mock.MagicMock(return_value=False)
         mock_context = mock.create_autospec(driver_context.PortContext)
         mock_context._plugin_context = mock.MagicMock()
         mock_context.current = {'binding:profile':
                                 {'local_link_information':
                                     [
                                         {
-                                            'switch_info': 'bar',
+                                            'switch_info': 'foo',
                                             'port_id': 2222
-                                        }
-                                    ]
-                                 },
+                                        },
+                                        {
+                                            'switch_info': 'foo',
+                                            'port_id': 3333
+                                        },
+                                    ]},
                                 'binding:vnic_type': 'baremetal',
                                 'id': '123'}
         mock_context.network.current = {
@@ -938,26 +1039,27 @@ class TestGenericSwitchDriver(unittest.TestCase):
                 'id': 123
             }
         ]
-        self.assertIsNone(driver.bind_port(mock_context))
-        self.assertFalse(mock_context.set_binding.called)
+
+        driver.bind_port(mock_context)
+        mock_context.set_binding.assert_not_called()
+        m_apc.assert_not_called()
         self.switch_mock.plug_port_to_network.assert_not_called()
-        self.assertFalse(m_apc.called)
 
     @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
-    def test_bind_port_with_different_physnet(self, m_apc, m_list):
+    def test_bind_port_with_physnet_port_not_supported(self, m_apc, m_list):
         driver = gsm.GenericSwitchDriver()
         driver.initialize()
+        driver._is_port_supported = mock.MagicMock(return_value=False)
         mock_context = mock.create_autospec(driver_context.PortContext)
         mock_context._plugin_context = mock.MagicMock()
         mock_context.current = {'binding:profile':
                                 {'local_link_information':
                                     [
                                         {
-                                            'switch_info': 'bar',
+                                            'switch_info': 'foo',
                                             'port_id': 2222
                                         }
-                                    ]
-                                 },
+                                    ]},
                                 'binding:vnic_type': 'baremetal',
                                 'id': '123'}
         mock_context.network.current = {
@@ -969,36 +1071,171 @@ class TestGenericSwitchDriver(unittest.TestCase):
                 'id': 123
             }
         ]
-        self.switch_mock._get_physical_networks.return_value = ['physnet2']
-        self.assertIsNone(driver.bind_port(mock_context))
-        self.assertFalse(mock_context.set_binding.called)
-        self.switch_mock.plug_port_to_network.assert_not_called()
-        self.assertFalse(m_apc.called)
+        self.switch_mock._get_physical_networks.return_value = ['physnet1']
 
-    def test_empty_methods(self, m_list):
+        driver.bind_port(mock_context)
+        mock_context.set_binding.assert_not_called()
+        m_apc.assert_not_called()
+        self.switch_mock.plug_port_to_network.assert_not_called()
+
+    @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
+    def test_bind_port_port_not_supported(self, m_apc, m_list):
         driver = gsm.GenericSwitchDriver()
         driver.initialize()
-        mock_context = mock.create_autospec(driver_context.NetworkContext)
-        mock_context.current = {'id': 22,
-                                'provider:network_type': 'vlan',
-                                'provider:segmentation_id': 22}
+        driver._is_port_supported = mock.MagicMock(return_value=False)
+        mock_context = mock.create_autospec(driver_context.PortContext)
+        mock_context._plugin_context = mock.MagicMock()
+        mock_context.current = {'binding:profile':
+                                {'local_link_information':
+                                    [
+                                        {
+                                            'switch_info': 'foo',
+                                            'port_id': 2222
+                                        }
+                                    ]},
+                                'binding:vnic_type': 'baremetal',
+                                'id': '123'}
+        mock_context.network.current = {
+            'provider:physical_network': 'physnet1'
+        }
+        mock_context.segments_to_bind = [
+            {
+                'segmentation_id': None,
+                'id': 123
+            }
+        ]
 
+        driver.bind_port(mock_context)
+        mock_context.set_binding.assert_not_called()
+        m_apc.assert_not_called()
+        self.switch_mock.plug_port_to_network.assert_not_called()
+
+    @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
+    def test_bind_portgroup_802_3ad_port_not_supported(self, m_apc, m_list):
+        driver = gsm.GenericSwitchDriver()
         driver.initialize()
+        driver._is_port_supported = mock.MagicMock(return_value=False)
+        mock_context = mock.create_autospec(driver_context.PortContext)
+        mock_context._plugin_context = mock.MagicMock()
+        mock_context.current = {'binding:profile':
+                                {'local_link_information':
+                                    [
+                                        {
+                                            'switch_info': 'foo',
+                                            'port_id': 2222
+                                        },
+                                        {
+                                            'switch_info': 'foo',
+                                            'port_id': 3333
+                                        },
+                                    ],
+                                    'local_group_information':
+                                    {
+                                        'bond_mode': '802.3ad'
+                                    }},
+                                'binding:vnic_type': 'baremetal',
+                                'id': '123'}
+        mock_context.network.current = {
+            'provider:physical_network': 'physnet1'
+        }
+        mock_context.segments_to_bind = [
+            {
+                'segmentation_id': None,
+                'id': 123
+            }
+        ]
 
-        driver.create_network_precommit(mock_context)
-        driver.update_network_precommit(mock_context)
-        driver.update_network_postcommit(mock_context)
-        driver.delete_network_precommit(mock_context)
-        driver.create_subnet_precommit(mock_context)
-        driver.create_subnet_postcommit(mock_context)
-        driver.update_subnet_precommit(mock_context)
-        driver.update_subnet_postcommit(mock_context)
-        driver.delete_subnet_precommit(mock_context)
-        driver.delete_subnet_postcommit(mock_context)
-        driver.create_port_precommit(mock_context)
-        driver.create_port_postcommit(mock_context)
-        driver.update_port_precommit(mock_context)
-        driver.delete_port_precommit(mock_context)
+        driver.bind_port(mock_context)
+        mock_context.set_binding.assert_not_called()
+        m_apc.assert_not_called()
+        self.switch_mock.plug_port_to_network.assert_not_called()
+        self.switch_mock.plug_bond_to_network.assert_not_called()
+
+        @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
+        def test_bind_port_unknown_switch(self, m_apc, m_list):
+            driver = gsm.GenericSwitchDriver()
+            driver.initialize()
+            mock_context = mock.create_autospec(driver_context.PortContext)
+            mock_context._plugin_context = mock.MagicMock()
+            mock_context.current = {'binding:profile':
+                                    {'local_link_information':
+                                        [
+                                            {
+                                                'switch_info': 'bar',
+                                                'port_id': 2222
+                                            }
+                                        ]},
+                                    'binding:vnic_type': 'baremetal',
+                                    'id': '123'}
+            mock_context.network.current = {
+                'provider:physical_network': 'physnet1'
+            }
+            mock_context.segments_to_bind = [
+                {
+                    'segmentation_id': None,
+                    'id': 123
+                }
+            ]
+            self.assertIsNone(driver.bind_port(mock_context))
+            self.assertFalse(mock_context.set_binding.called)
+            self.switch_mock.plug_port_to_network.assert_not_called()
+            self.assertFalse(m_apc.called)
+
+        @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
+        def test_bind_port_with_different_physnet(self, m_apc, m_list):
+            driver = gsm.GenericSwitchDriver()
+            driver.initialize()
+            mock_context = mock.create_autospec(driver_context.PortContext)
+            mock_context._plugin_context = mock.MagicMock()
+            mock_context.current = {'binding:profile':
+                                    {'local_link_information':
+                                        [
+                                            {
+                                                'switch_info': 'bar',
+                                                'port_id': 2222
+                                            }
+                                        ]},
+                                    'binding:vnic_type': 'baremetal',
+                                    'id': '123'}
+            mock_context.network.current = {
+                'provider:physical_network': 'physnet1'
+            }
+            mock_context.segments_to_bind = [
+                {
+                    'segmentation_id': None,
+                    'id': 123
+                }
+            ]
+            self.switch_mock._get_physical_networks.return_value = ['physnet2']
+            self.assertIsNone(driver.bind_port(mock_context))
+            self.assertFalse(mock_context.set_binding.called)
+            self.switch_mock.plug_port_to_network.assert_not_called()
+            self.assertFalse(m_apc.called)
+
+        def test_empty_methods(self, m_list):
+            driver = gsm.GenericSwitchDriver()
+            driver.initialize()
+            mock_context = mock.create_autospec(driver_context.NetworkContext)
+            mock_context.current = {'id': 22,
+                                    'provider:network_type': 'vlan',
+                                    'provider:segmentation_id': 22}
+
+            driver.initialize()
+
+            driver.create_network_precommit(mock_context)
+            driver.update_network_precommit(mock_context)
+            driver.update_network_postcommit(mock_context)
+            driver.delete_network_precommit(mock_context)
+            driver.create_subnet_precommit(mock_context)
+            driver.create_subnet_postcommit(mock_context)
+            driver.update_subnet_precommit(mock_context)
+            driver.update_subnet_postcommit(mock_context)
+            driver.delete_subnet_precommit(mock_context)
+            driver.delete_subnet_postcommit(mock_context)
+            driver.create_port_precommit(mock_context)
+            driver.create_port_postcommit(mock_context)
+            driver.update_port_precommit(mock_context)
+            driver.delete_port_precommit(mock_context)
 
 
 class TestGenericSwitchDriverStaticMethods(unittest.TestCase):
